<?
/**
 * @package rbcore
 * File:  fmt.php
 * Formatting functions
 * Author:  Robocommerce, LLC (Zvi Boshernitzan and Brian Rosenthal)
 * Copyright 2004, 2005:  Robocommerce, LLC (http://www.robocommerce.com)
 * Distributed under Gnu General Public License (see includes/gpl_general.txt)
 * Version 1.0
 */


/**
 * various string formatting operations.
 * 
 * @package rbcore
 * @static
 */
class fmt {
	/**
	 * @deprecated (php's native str_replace does this already)
	 */
	function str_replace($arr, $s) {
		foreach ($arr as $key => $value) {
			$s = str_replace($key, $value, $s);
		}
		return $s;
	}
	/**
	 * @todo this should go into rbform or something.  not a formatting command.
	 */
	function textarea_rows($s) {
		$arr = split("\r", $s);
		$extra_lines = 0;
		foreach ($arr as $s) {
			$extra_lines += (int)(strlen($s) / 80);
		}
		return max(5, count($arr) + $extra_lines + 5);
	}


	function clean_constant_name($name,$prefix="",$wordsep=" ") {
		$name = str_replace($prefix,"",$name);
		return strtolower(str_replace("_" ,$wordsep,$name));
	}

	function pluralize($s) {
		if (strtolower($s) == strtolower("person"))	return "people";
		if (substr($s, strlen($s) - 1, 1) == "s")	return $s . "es";
		if (substr($s, strlen($s) - 2, 2) == "sh")	return $s . "es";
		if (substr($s, strlen($s) - 2, 2) == "ch")  return $s . "es";
		if (substr($s, strlen($s) - 1, 1) == "y")	return substr($s, 0, strlen($s) - 1) . "ies";
		return $s . "s";
	}



	function _init_overlib() {
		static $header = '
<div id="overDiv" style="position:absolute; visibility:hidden; z-index:1000;"></div>
<script language="JavaScript" src="/js/overlib.js"></script>
<style type="text/css">
.help {
	padding: 0px;
	padding-right: 7px;
	background-image: url(/admin/images/icons/bullet_help.gif);
	background-repeat: no-repeat;
	background-position: top right;
}</style>
';
		echo $header;
	}
	function help_overlib($label_text,$help_text) {
		if (empty($help_text)) return $label_text;
		$params = func_get_args();
		$params = array_slice($params,2);
		$help_text = trim($help_text);
		$len = strlen($help_text);
		$params = array_unique($params);
		$p = (count($params) ? ','.join(',',$params) : '');
		$onmouseover = fmt::js_string(($len > 1000 ? 'Click to view all' : $help_text));
		$onclick = fmt::js_string(($len > 1000 ? $help_text : ''));
		return (
			'<span class="help"'
			. ((strlen($onmouseover)>2)		? ' onmouseover="return overlib(' . $onmouseover . ",WIDTH,300" . $p . ');" ' : '')
			. ((strlen($onclick)>2)			? ' onclick="return overlib(' . $onclick . ',WIDTH,300,STICKY' . $p . ');" ' : '')
			. ' onmouseout="return nd();">' . $label_text . '</span>'
			);
	}


	////////////////////////////////////// fmt: variables, debugging ///////

	/**
	 * like print_r, except returns instead of echoing.
	 * print_r in php >4.3.0 does this automatically.
	 */
	function dump($var) {
		ob_start();
		print_r($var);
		$ret_str = ob_get_contents();
		ob_end_clean();
		return $ret_str;
	}
	function assert_debug() {
		$args = func_get_args(); 
		$arr = array();
		foreach ($args as $var) {
			$arr[] = fmt::debug($var);
		}
		return '/* ' . join(",", $arr) . ' */';
	}
	function debug($var) {return '<pre>' . fmt::dump($var) . '</pre>';}

	/**
	 * text-only version of {@link bt::bt()} for emails.
	 * @param int $INDENT offset of page/line number markers from left margin.
	 * @return string text-only backtrace ({@link fmt::bt()} has html)
	 */
	function btt($INDENT=50) {
		$MAXSTRLEN = 64;
		$BASE = $_SERVER['RB_RELEASE_ROOT'];
		$s = "---backtrace---\n" . 'BASE: ' . $BASE . "\n\n" ;
		$traceArr = debug_backtrace();
		array_shift($traceArr);
		$tabs = sizeof($traceArr)-1;

		foreach($traceArr as $arr) {
			$nl = "";
			for ($i=0; $i < $tabs; $i++) $nl .= '   ';
			$tabs -= 1;
			if (isset($arr['class'])) $nl .= $arr['class'].'.';
			$args = array();
			if(!empty($arr['args'])) foreach($arr['args'] as $v) {
				if (is_null($v)) $args[] = 'null';
				else if (is_array($v)) $args[] = 'Array['.sizeof($v).']';
				else if (is_object($v)) $args[] = 'Object:'.get_class($v);
				else if (is_bool($v)) $args[] = $v ? 'true' : 'false';
				else {
					$v = (string) @$v;
					$str = htmlspecialchars(substr($v,0,$MAXSTRLEN));
					if (strlen($v) > $MAXSTRLEN) $str .= '...';
					$args[] = "\"".$str."\"";
				}
			}
			
			$nl .= $arr['function'].'('.implode(', ',$args).')';
			$Line = (isset($arr['line'])? $arr['line'] : "unknown");
			$File = (isset($arr['file'])? $arr['file'] : "unknown");
			$File = str_replace($BASE,'',$File);
			
			
			$len = $INDENT - strlen($nl); 
			if ($len < 0) {
				$s .= $nl . "\n" . str_repeat(' ',$INDENT);
			} else {
				$s .= $nl . str_repeat(' ',$len);
			} 
			$s .= sprintf("# line %4d, file: %s", $Line, $File, $File);
			$s .= "\n";
		}   
		$s .= '---end backtrace---';
		return $s;
	}

	function debug_overlib($v,$lbl='') {
		$vdbg = preg_replace("/\n+/","<br>",fmt::debug($v));
		$vdbg = str_replace('"','&quot;',$vdbg);
		$MAXSTRLEN = 20;
		if (is_null($v)) return 'null';
		elseif (is_array($v))  return fmt::help_overlib($lbl . 'Array['.sizeof($v).']',$vdbg);
		elseif (is_object($v)) return fmt::help_overlib($lbl . 'Object:'.get_class($v),$vdbg);
		elseif (is_bool($v))   return ($v ? 'true' : 'false');
		else {
			$v = (string) @$v;
			$str = htmlspecialchars(substr($v,0,$MAXSTRLEN));
			if (strlen($v) > $MAXSTRLEN) {
				$str .= '...';
				$str = fmt::help_overlib($lbl . $str,$vdbg);
			}
			return "\"".$str."\"";
		}
		
	}

	/**
	 * @return string an html backtrace {@link fmt::btt()} has text-only for emails, etc.
	 * @see fmt::btt
	 */ 
	function bt() {
		$s = '<pre align=left style="font-size: 8pt;">';
		$traceArr = debug_backtrace();
		array_shift($traceArr);
		$tabs = sizeof($traceArr)-1;
		foreach($traceArr as $arr) {
			for ($i=0; $i < $tabs; $i++) $s .= '&nbsp;';
			$tabs -= 1;
			$s .= '<font face="Courier New,Courier">';
			if (isset($arr['class'])) $s .= $arr['class'].'.';
			$args = array();
			if(!empty($arr['args'])) 
				$args = array_map(array('fmt','debug_overlib'),$arr['args']);
			$s .= $arr['function'].'('.implode(', ',$args).')</font>';
			$Line = (isset($arr['line'])? $arr['line'] : "unknown");
			$File = (isset($arr['file'])? $arr['file'] : "unknown");
			$s .= sprintf('<font color="#808080"> # line <b>%4d</b>, file: <a href="file:/%s">%s</a></font>', $Line, $File, $File);
			$s .= "\n";
		}   
		$s .= '</pre>';
		return $s;
	}

	////////////////////////////////////// fmt: numbers, money ///////////

	/**
	 * format a number $x into scientific notation
	 */
	function sci($x, $d=-1) {
		$min=($x<0)?"-":"";
		$x=abs($x);  
		$e=floor(($x!=0)?log10($x):0);
		$x*=pow(10,-$e);
		$fmt=($d>=0)?".".$d:"";
		$e=($e>=0)?"+".sprintf("%02d",$e):"-".sprintf("%02d",-$e);
		return sprintf("$min%".$fmt."fe%s",$x,$e);
	}

	/**
	 * @param string $s a formatted dollar amount (e.g. $23.22).
	 * @return float number with currency markers stripped. (e.g. 23.22000001)
	 */
	function strip_currency($s) {
		$s = preg_replace('/\$/', "", $s);
		$s = preg_replace('/[^0-9\.()$-]/','',$s);
		$matches = array();
		if (preg_match("/\((.*)\)/", $s, $matches))  {
			$s = $matches[1];
			$s = -1.0 * (float)$s;
		}
		return (float)$s;
	}
	function currency_value($s) {
		return 1;
	}
	/**
	 * format a number as a currency.
	 * @param float $n amount to format.
	 * @return string formatted as currency (parentheses for negative amounts).
	 */
	function currency($n) {
		if ($n >= 0) return "$" . number_format($n		, 2, ".", ",");
		return "($"				. number_format(-1 * $n	, 2, ".", ",") . ")";
	}
	function round_price($p)	{return fmt::round($p, 2);}
	function round($f, $n)		{return sprintf("%." . $n . "f", $f);}
	function rb_round($f, $n)	{return fmt::round($f, $n);}
	function tax_value($f, $padding = TAX_DECIMAL_PLACES) {
		return fmt::round($f, $padding);
	}
	function percent($n) {
		return (int)$n . '%';
	}
	/**
	 * alias for {@link fmt::strip_currency()}.
	 * @deprecated
	 */
	function interpret_currency($s) {
		return fmt::strip_currency($s);
	}


	/////////////////////////////////// fmt: dates ////////////////////

	/** 
	 * wrapper for date
	 *
	 * handles input dates from databases, such as mysql's timestamps
	 * and formats those as iso dates, or some other date() compatible
	 * format such as below examples
	 *
	 * date() compatible format examples:
	 * <pre>
	 *  - 'F j, Y, g:i a'                -> March 10, 2001, 5:16 pm
	 *  - 'm.d.y'                        -> 03.10.01
	 *  - 'j, n, Y'                      -> 10, 3, 2001
	 *  - 'Ymd'                          -> 20010310
	 *  - 'h-i-s, j-m-y, it is w Day z ' -> 05-16-17, 10-03-01, 1631 1618 6 Fripm01
	 *  - '\i\t \i\s \t\h\e jS \d\a\y.'  -> It is the 10th day.
	 *  - 'D M j G:i:s T Y'              -> Sat Mar 10 15:16:08 MST 2001
	 *  - 'H:m:s \m \i\s\ \m\o\n\t\h'    -> 17:03:17 m is month
	 *  - 'H:i:s'                        -> 17:16:17
	 *  - "M d 'y"                       -> Mar 10 '01
	 * </pre>
	 */
	function iso_date($the_date=null,$format=null) {
		if (empty($the_date)) $the_date = date("Y-m-d H:i:s");
		$format = coalesce($format,"Y-m-j");
		if (is_numeric($the_date)) {
			if ( abs(time() - $the_date) < (time()/4) ) {
				return date($format,$the_date);
			}
		}
		if ( ereg("([0-9]{4})-?([0-9]{2})-?([0-9]{2}) ?([0-9]{2})?:?([0-9]{2})?:?([0-9]{2})?[0-9]*"
				  , $the_date, $regs)) {
			return date($format, mktime($regs[4],$regs[5],$regs[6],$regs[2],$regs[3],$regs[1]));
		}
		return "$the_date";
	}
	/**
	 * a wrapper for {@link fmt::iso_date()} that recognized pre-defined formats:
	 * 
	 * @todo handle more input formats
	 * @param mixed $the_date a date acceptable to {@link fmt::iso_date()}
	 * @param string $format is a key in:
	 *   <code>
	 *	 static $formats = array(
	 *		"admin"			=> "M d 'y, g:i a"
	 *		, "tinylong"	=> "M d 'y, g:i a"
	 *		, "admin_short"	=> "M d 'y"
	 *		, "tiny"		=> "M d 'y"
	 *		, "long"		=> 'F j, Y g:i a'
	 *		, "short"		=> 'm/d/Y'
	 *		, "datetime"	=> 'm/d/Y H:i:s'
	 *		, "iso"			=> 'Y-m-j'
	 *		);
	 *   </code>
	 */
	function date($the_date,$format=null) {
		static $formats = array(
			"admin"			=> "M d 'y, g:i a"
			, "tinylong"	=> "M d 'y, g:i a"
			, "admin_short"	=> "M d 'y"
			, "tiny"		=> "M d 'y"
			, "long"		=> 'F j, Y g:i a'
			, "short"		=> 'm/d/Y'
			, "datetime"	=> 'm/d/Y H:i:s'
			, "iso"			=> 'Y-m-j'
			);
		return fmt::iso_date($the_date , coalesce($formats[$format],$format));
	}


	/** 
	 * a simpler/faster {@link fmt::iso_date()} good for mysql timestamps.
	 * @param string $d mysql timestamp or string starting with date as YYYY-MM-DD.
	 */
	function sql_date($d) {return substr($d, 0, 10);}

	/**
	 * replaces brace-delimited keys belonging to array $row in $tpl with their values.
	 * @param string $tpl string with brace-delimited keys (e.g. "{last}, {first} {middle}").
	 * @param array $row array containing values to replace (e.g. array("first"=>"John","last"=>"Doe") ).
	 * @return string having keys replaced with values (e.g. "John, Doe {middle}").
	 */
	function tpl($tpl,$row) {
		$row = coalesce($row,array());
		foreach($row as $key => $value) {
			if (!empty($key) and !is_numeric($key)) {
				$tpl = str_replace('{'.$key.'}',$value,$tpl);
			}
		}
		return $tpl;
	}

	function truncate($string, $length = 80, $etc = '...', $break_words = false) {
		if ($length == 0) return '';
		if (strlen($string) > $length) {
			$length -= strlen($etc);
			if (!$break_words) $string = preg_replace('/\s+?(\S+)?$/', '', substr($string, 0, $length+1));
			return substr($string, 0, $length).$etc;
		} else return $string;
	}

	function camel_to_underscore($s) {
		return preg_replace('/(?<=[a-z])[A-Z]|(?<!^)[A-Z](?=[a-z])/',' \0', $s);
	}
	function filename_to_title($s) {
		$s = preg_replace("/\.html/", "", $s);
		$s = preg_replace("/-/", " ", $s);
		$arr = split(" ", $s); $arr2 = array();
		foreach ($arr as $word) {
			$f = substr($word, 0, 1);
			$rest = substr($word, 1, strlen($word) - 1);
			$word = strtoupper($f) . $rest;
			$arr2[] = $word;
		}
		$s = join(" ", $arr2);
		return $s;
	}
	
	function urlkeywords($s,$separate_studlycaps=false) {
		if ($separate_studlycaps) $s = fmt::camel_to_underscore($s);
		$s = strtolower($s);
		$s = preg_replace("/<[^>]+?>/", "", $s);          // strip tags
		$s = preg_replace('/\.+/','.',$s);                // rm repeated '.'
		$s = preg_replace('/[^a-zA-Z0-9.]+/','-',$s);     // create '-' between words
		$s = preg_replace('/(^[^a-zA-Z0-9.]+)|([^a-zA-Z0-9.]+$)/','',$s); // strip junk at start/end
		return $s;
	}
	
	/**
	 *  tries to resolve all the segments (that need to be resolved - it
	 *  ignores ones with empty or non-numerical values) and create a
	 *  pure-segment-driven url.
	 *
	 *  If it cannot resolve any of the segments, then it addes the args
	 *  as url parameters to a base url and returns a conventional url.
	 *
	 * @see url_segment_management
	 */
	function seg_url($args,$base_url = "index.php") {
		$segments = url_segment_management::create_segments($args);
		if ($segments === false) {
			return rburl::add_params($base_url,$args);
		}
		return '/' . url_segment_management::create_segments($args) . '.htm';
	}


	///////////////////////////// fmt: html formatting ///////////////////////

	/**
	 * remove all html tags from $s.
	 * @param string $s html text
	 * @return string text without html tags
	 */
	function strip_tags($s) { return preg_replace("/<[^>]+?>/", " ", $s); }

	/** 
	 * here we may need some special function to escape values to be
	 * sent to html attributes
	 */
	function html_attrs($dict) {
		if (!is_array($dict)) return '';
		$attrs = '';
		foreach($dict as $key => $val) {
			if (is_array($val)) {
				$new_val = rba::add_prefix($key.'_',$val);
				$attr .= ' ' . fmt::html_attrs($new_val);
				continue;
			}
			if (is_numeric($key)) { $attrs .= ' ' . $val; continue; }
			$attrs .= ' ' . $key . '="' . htmlspecialchars($val) . '"';
		}
		return $attrs;
	}

	/**
	 * safely build html tags.
	 * @param string $name tag name
	 * @param array @attrs array of html attributes.
	 * usage:
	 * <code>return fmt::tag('input',compact('type','name','value'));</code>
	 */
	function tag($name) {
		$attrs = rba::__slice(func_get_args(),1);
		return '<' . $name . fmt::html_attrs($attrs) . '>';
	}

	/**
	 * create an input type="hidden" element.
	 * @param string $name
	 * @param string $value
	 */
	function hidden_var($name, $value = '') {
		return '<input type="hidden"' . fmt::html_attrs(compact('name','value')) . '>';
	}

	/**
	 * create a string of many input type="hidden" elements
	 * @param arr
	 *    one of
	 *      - array( name1=>value1, name2=>value2, ...)
	 *      - array( array(name1,value1), array(name2,value2), ...)
	 * @return str
	 */
	function hidden_vars($pairs) {
		$vars = "";
		if (is_array($pairs)) {
			foreach($pairs as $name => $val) {
				if (is_array($val)) {$name = $val[0]; $val = $val[1];}
				$vars .= fmt::hidden_var($name,$val) . "\n";
			}
		}
		return $vars;
	}
	
	function encrypt_password($plain) {
		$s = '';
		for ($i=0; $i<10; $i++) $s .= rbrandom::number();
		$salt = substr(md5($s), 0, 2);
		return md5($salt . $plain) . ':' . $salt;
	}
	

	///// fmt: javascript stuff //////////////////////////////////////////////
	
	function js_bool($val) {
		return (empty($val) ? 'false' : 'true');
	}

	function js_string($val) {
		if (!is_numeric($val)) return "'".addslashes($val)."'";
		else return $val;
	}
	function js_array($arr) {
		foreach ($arr as $val) $parts[] = fmt::js_string($val);
		return "[" . join(" , " , $parts) . "]";
	}
	function js_dict($arr) {
		$parts = array();
		foreach ($arr as $key => $val) {
			$parts[] = fmt::js_string($key) . " : " . fmt::js_string($val);
		}
		return "{" . join(" , ",$parts) . "}";
	}

	

}

?>
